<?php

/**
 * @file
 * Defines default request handler class
 */

/**
 * Default request handler class for personalized fragments.
 *
 * This class is responsible for handling a request and returning the result
 * along with appropriate HTTP headers back to the client.
 */
class AuthcacheP13nDefaultRequestHandler {
  const PER_USER = 0x0002;
  const PER_PAGE = 0x0004;

  /**
   * @var AuthcacheP13nCoreService
   */
  protected $coreService;

  /**
   * @var AuthcacheP13nRequestValidator
   */
  protected $requestValidator;

  /**
   * @var AuthcacheP13nContentBuilder
   */
  protected $contentBuilder;

  /**
   * @var AuthcacheP13nContentEncoder
   */
  protected $contentEncoder;

  /**
   * Bootstrap phase required to serve this request.
   */
  protected $bootstrapPhase;

  /**
   * Number of seconds the response should be cached in the users browser.
   */
  protected $cacheMaxage;

  /**
   * Set to TRUE when the response is identical for all users.
   */
  protected $cacheGranularity;

  /**
   * Construct a new instance.
   */
  public function __construct(
    AuthcacheP13nCoreServiceInterface $core_service,
    AuthcacheP13nRequestValidatorInterface $request_validator,
    AuthcacheP13nContentBuilderInterface $content_builder,
    AuthcacheP13nContentEncoderInterface $content_encoder,
    $cache_maxage,
    $cache_granularity,
    $bootstrap_phase
  ) {
    $this->coreService = $core_service;
    $this->requestValidator = $request_validator;
    $this->contentBuilder = $content_builder;
    $this->contentEncoder = $content_encoder;
    $this->cacheMaxage = $cache_maxage;
    $this->cacheGranularity = $cache_granularity;
    $this->bootstrapPhase = $bootstrap_phase;
  }

  /**
   * {@inheritdoc}
   */
  public function handle($params) {
    $this->addVaryHeader();

    $input = $this->validate($params);
    $this->bootstrap();
    $result = $this->build($input);
    $this->addCacheControlHeader();
    print $this->encode($result);
  }

  /**
   * {@inheritdoc}
   */
  public function defaults() {
    url('', array('prefix' => &$prefix));
    $args = array(
      'q' => empty($prefix) ? '' : $prefix,
    );

    // Add path if per-page caching is required.
    if ($this->cacheGranularity & AuthcacheP13nDefaultRequestHandler::PER_PAGE) {
      $args['q'] .= $_GET['q'];
    }

    return $args;
  }

  /**
   * Add Vary: Cookie header if necessary.
   */
  protected function addVaryHeader() {
    // Add Vary: Cookie header if necessary.
    if ($this->cacheGranularity & AuthcacheP13nDefaultRequestHandler::PER_USER) {
      $this->coreService->drupalAddHttpHeader('Vary', 'Cookie');
    }
  }

  /**
   * Add Cache-Control header.
   */
  protected function addCacheControlHeader() {
    if ($this->cacheMaxage) {
      $this->coreService->drupalAddHttpHeader('Cache-Control', 'public, max-age=' . $this->cacheMaxage);
    }
    else {
      $this->coreService->drupalAddHttpHeader('Cache-Control', 'no-cache, must-revalidate, post-check=0, pre-check=0');
    }
  }

  /**
   * Bootstrap to the required level if necessary.
   */
  protected function bootstrap() {
    if ($this->bootstrapPhase) {
      $this->coreService->drupalBootstrap($this->bootstrapPhase);
    }
  }

  /**
   * Return validated input parameters.
   *
   * @throws: AuthcacheP13nRequestInvalidInput
   */
  protected function validate($params) {
    return $this->requestValidator ? $this->requestValidator->validate($params) : $params;
  }

  /**
   * Construct response from the validated input.
   */
  protected function build($input) {
    return $this->contentBuilder->build($input);
  }

  /**
   * Encode the result returned by ::build().
   */
  protected function encode($result) {
    $this->coreService->drupalAddHttpHeader('Content-Type', $this->contentEncoder->contentType());
    return $this->contentEncoder->encode($result);
  }
}
